@@ -255,7 +255,7 @@ std::string video_frame::format_name() const
 
 std::string video_frame::format_info() const
 {
-    return str::asprintf("%dx%d, %.3g:1", width, height, aspect_ratio);
+    return str::asprintf("%dx%d, %.3g:1", raw_width, raw_height, aspect_ratio);
 }
 
 static int next_multiple_of_4(int x)
@@ -353,12 +353,19 @@ void video_frame::copy_plane(int view, int plane, void *buf) const
         break;
     }
 
-    size_t dst_offset = 0;
-    for (size_t y = 0; y < lines; y++)
+    if (src_row_size == dst_row_size)
     {
-        std::memcpy(dst + dst_offset, src + src_offset, dst_row_width);
-        dst_offset += dst_row_size;
-        src_offset += src_row_size;
+        std::memcpy(dst, src + src_offset, lines * src_row_size);
+    }
+    else
+    {
+        size_t dst_offset = 0;
+        for (size_t y = 0; y < lines; y++)
+        {
+            std::memcpy(dst + dst_offset, src + src_offset, dst_row_width);
+            dst_offset += dst_row_size;
+            src_offset += src_row_size;
+        }
     }
 }
 
@@ -400,21 +407,23 @@ std::string audio_blob::format_name() const
 
 int audio_blob::sample_bits() const
 {
+    int bits = 0;
     switch (sample_format)
     {
     case u8:
-        return 8;
+        bits = 8;
         break;
     case s16:
-        return 16;
+        bits = 16;
         break;
     case f32:
-        return 32;
+        bits = 32;
         break;
     case d64:
-        return 64;
+        bits = 64;
         break;
     }
+    return bits;
 }
 
 parameters::parameters() :
