@@ -135,6 +135,7 @@ struct ffmpeg_stuff
     std::vector<AVCodec *> video_codecs;
     std::vector<std::deque<AVPacket> > video_packet_queues;
     std::vector<mutex> video_packet_queue_mutexes;
+    std::vector<AVPacket> video_packets;
     std::vector<video_decode_thread> video_decode_threads;
     std::vector<AVFrame *> video_frames;
     std::vector<AVFrame *> video_out_frames;
@@ -616,6 +617,8 @@ void media_object::open(const std::string &url)
             _ffmpeg->video_frame_templates.push_back(video_frame());
             set_video_frame_template(j);
             // Allocate things required for decoding
+            _ffmpeg->video_packets.push_back(AVPacket());
+            av_init_packet(&(_ffmpeg->video_packets[j]));
             _ffmpeg->video_decode_threads.push_back(video_decode_thread(_url, _ffmpeg, j));
             _ffmpeg->video_frames.push_back(avcodec_alloc_frame());
             if (!_ffmpeg->video_frames[j])
@@ -938,34 +941,52 @@ void read_thread::run()
         {
             if (packet.stream_index == _ffmpeg->video_streams[i])
             {
+                // We do not check for missing timestamps here, as we do with audio
+                // packets, for the following reasons:
+                // 1. The video decoder might fill in a timestamp for us
+                // 2. We cannot drop video packets anyway, because of their
+                //    interdependencies. We would mess up decoding.
                 if (av_dup_packet(&packet) < 0)
                 {
                     throw exc(_url + ": Cannot duplicate packet.");
                 }
                 _ffmpeg->video_packet_queue_mutexes[i].lock();
                 _ffmpeg->video_packet_queues[i].push_back(packet);
+                _ffmpeg->video_packet_queue_mutexes[i].unlock();
+                packet_queued = true;
                 msg::dbg(_url + ": "
                         + str::from(_ffmpeg->video_packet_queues[i].size())
                         + " packets queued in video stream " + str::from(i) + ".");
-                _ffmpeg->video_packet_queue_mutexes[i].unlock();
-                packet_queued = true;
             }
         }
         for (size_t i = 0; i < _ffmpeg->audio_streams.size() && !packet_queued; i++)
         {
             if (packet.stream_index == _ffmpeg->audio_streams[i])
             {
-                if (av_dup_packet(&packet) < 0)
-                {
-                    throw exc(_url + ": Cannot duplicate packet.");
-                }
                 _ffmpeg->audio_packet_queue_mutexes[i].lock();
-                _ffmpeg->audio_packet_queues[i].push_back(packet);
-                msg::dbg(_url + ": "
-                        + str::from(_ffmpeg->audio_packet_queues[i].size())
-                        + " packets queued in audio stream " + str::from(i) + ".");
+                if (_ffmpeg->audio_packet_queues[i].empty()
+                        && _ffmpeg->audio_last_timestamps[i] == std::numeric_limits<int64_t>::min()
+                        && packet.dts == static_cast<int64_t>(AV_NOPTS_VALUE))
+                {
+                    // We have no packet in the queue and no last timestamp, probably
+                    // because we just seeked. We *need* a packet with a timestamp.
+                    msg::dbg(_url + ": audio stream " + str::from(i)
+                            + ": dropping packet because it has no timestamp");
+                }
+                else
+                {
+                    if (av_dup_packet(&packet) < 0)
+                    {
+                        _ffmpeg->audio_packet_queue_mutexes[i].unlock();
+                        throw exc(_url + ": Cannot duplicate packet.");
+                    }
+                    _ffmpeg->audio_packet_queues[i].push_back(packet);
+                    packet_queued = true;
+                    msg::dbg(_url + ": "
+                            + str::from(_ffmpeg->audio_packet_queues[i].size())
+                            + " packets queued in audio stream " + str::from(i) + ".");
+                }
                 _ffmpeg->audio_packet_queue_mutexes[i].unlock();
-                packet_queued = true;
             }
         }
         if (!packet_queued)
@@ -998,7 +1019,6 @@ int64_t video_decode_thread::handle_timestamp(int64_t timestamp)
 
 void video_decode_thread::run()
 {
-    AVPacket pkt;
     int frame_finished = 0;
     do
     {
@@ -1021,14 +1041,15 @@ void video_decode_thread::run()
             }
         }
         while (empty);
+        av_free_packet(&(_ffmpeg->video_packets[_video_stream]));
         _ffmpeg->video_packet_queue_mutexes[_video_stream].lock();
-        pkt = _ffmpeg->video_packet_queues[_video_stream].front();
+        _ffmpeg->video_packets[_video_stream] = _ffmpeg->video_packet_queues[_video_stream].front();
         _ffmpeg->video_packet_queues[_video_stream].pop_front();
         _ffmpeg->video_packet_queue_mutexes[_video_stream].unlock();
         _ffmpeg->reader->start();       // Refill the packet queue
         avcodec_decode_video2(_ffmpeg->video_codec_ctxs[_video_stream],
-                _ffmpeg->video_frames[_video_stream], &frame_finished, &pkt);
-        av_free_packet(&pkt);
+                _ffmpeg->video_frames[_video_stream], &frame_finished,
+                &(_ffmpeg->video_packets[_video_stream]));
     }
     while (!frame_finished);
 
@@ -1055,9 +1076,24 @@ void video_decode_thread::run()
         _frame.line_size[0][2] = _ffmpeg->video_frames[_video_stream]->linesize[2];
     }
 
-    _frame.presentation_time = handle_timestamp(pkt.dts * 1000000
-            * _ffmpeg->format_ctx->streams[_ffmpeg->video_streams[_video_stream]]->time_base.num
-            / _ffmpeg->format_ctx->streams[_ffmpeg->video_streams[_video_stream]]->time_base.den);
+    if (_ffmpeg->video_packets[_video_stream].dts != static_cast<int64_t>(AV_NOPTS_VALUE))
+    {
+        _frame.presentation_time = handle_timestamp(_ffmpeg->video_packets[_video_stream].dts * 1000000
+                * _ffmpeg->format_ctx->streams[_ffmpeg->video_streams[_video_stream]]->time_base.num
+                / _ffmpeg->format_ctx->streams[_ffmpeg->video_streams[_video_stream]]->time_base.den);
+    }
+    else if (_ffmpeg->video_last_timestamps[_video_stream] != std::numeric_limits<int64_t>::min())
+    {
+        msg::wrn(_url + ": video stream " + str::from(_video_stream)
+                + ": no timestamp available, using a questionable guess");
+        _frame.presentation_time = _ffmpeg->video_last_timestamps[_video_stream];
+    }
+    else
+    {
+        msg::wrn(_url + ": video stream " + str::from(_video_stream)
+                + ": no timestamp available, using a bad guess");
+        _frame.presentation_time = _ffmpeg->pos;
+    }
 }
 
 void media_object::start_video_frame_read(int video_stream)
@@ -1134,7 +1170,7 @@ void audio_decode_thread::run()
             _ffmpeg->audio_packet_queues[_audio_stream].pop_front();
             _ffmpeg->audio_packet_queue_mutexes[_audio_stream].unlock();
             _ffmpeg->reader->start();   // Refill the packet queue
-            if (timestamp == std::numeric_limits<int64_t>::min())
+            if (timestamp == std::numeric_limits<int64_t>::min() && packet.dts != static_cast<int64_t>(AV_NOPTS_VALUE))
             {
                 timestamp = packet.dts * 1000000
                     * _ffmpeg->format_ctx->streams[_ffmpeg->audio_streams[_audio_stream]]->time_base.num
@@ -1168,6 +1204,16 @@ void audio_decode_thread::run()
             av_free_packet(&packet);
         }
     }
+    if (timestamp == std::numeric_limits<int64_t>::min())
+    {
+        timestamp = _ffmpeg->audio_last_timestamps[_audio_stream];
+    }
+    if (timestamp == std::numeric_limits<int64_t>::min())
+    {
+        msg::wrn(_url + ": audio stream " + str::from(_audio_stream)
+                + ": no timestamp available, using a bad guess");
+        timestamp = _ffmpeg->pos;
+    }
 
     _blob = _ffmpeg->audio_blob_templates[_audio_stream];
     _blob.data = _ffmpeg->audio_blobs[_audio_stream].ptr();
@@ -1240,6 +1286,14 @@ void media_object::seek(int64_t dest_pos)
         _ffmpeg->audio_packet_queues[i].clear();
     }
     // The next read request must update the position
+    for (size_t i = 0; i < _ffmpeg->video_streams.size(); i++)
+    {
+        _ffmpeg->video_last_timestamps[i] = std::numeric_limits<int64_t>::min();
+    }
+    for (size_t i = 0; i < _ffmpeg->audio_streams.size(); i++)
+    {
+        _ffmpeg->audio_last_timestamps[i] = std::numeric_limits<int64_t>::min();
+    }
     _ffmpeg->pos = std::numeric_limits<int64_t>::min();
     // Restart packet reading
     _ffmpeg->reader->reset();
@@ -1300,6 +1354,10 @@ void media_object::close()
             av_free_packet(&_ffmpeg->video_packet_queues[i][j]);
         }
     }
+    for (size_t i = 0; i < _ffmpeg->video_packets.size(); i++)
+    {
+        av_free_packet(&(_ffmpeg->video_packets[i]));
+    }
     for (size_t i = 0; i < _ffmpeg->audio_codec_ctxs.size(); i++)
     {
         if (i < _ffmpeg->audio_codecs.size() && _ffmpeg->audio_codecs[i])
